from django.urls import path
from rest_framework.routers import DefaultRouter

from drf_auth_service.authentication.views import EBSTokenRefreshView, TokenByCookieView
from drf_auth_service.settings import settings
from drf_auth_service.socials.views import social_login

router = DefaultRouter(trailing_slash=False)
router.register(r'services', settings.VIEWS.SERVICE_VIEWS, basename='service')
router.register(r'users', settings.VIEWS.USER_VIEWS, basename='user')
router.register(r'authentication', settings.VIEWS.AUTHENTICATION_VIEWS, basename='authentication')

urlpatterns = router.urls + [
    path('authentication/login', settings.VIEWS.AUTHENTICATION_LOGIN_VIEWS.as_view(), name='authentication-login'),
    path('authentication/oauth', social_login, name='social_login'),
    path('authentication/refresh', EBSTokenRefreshView.as_view(), name='authentication-refresh'),
    path('authentication/cookie', TokenByCookieView.as_view(), name='cookie-refresh'),
]



from django.db.models.signals import post_save
from django.dispatch import receiver

from drf_auth_service.models import Service, Config
from drf_auth_service.settings import ObjDict, SETTINGS_TO_IMPORT_IN_DB, settings


@receiver(post_save, sender=Service)
def service_post_save(sender, instance, created, **kwargs):
    if created:
        configs = []

        for key, val in settings._wrapped.__dict__.items():
            if key in SETTINGS_TO_IMPORT_IN_DB:
                if type(val) == ObjDict or type(val) == dict:

                    for dict_key, dict_value in val.items():
                        if not type(dict_value) == list:
                            configs.append(Config(
                                type='str',
                                value=dict_value,
                                key=dict_key,
                                service=instance
                            ))

                elif type(val) == list:
                    value = ''
                    for list_value in val:
                        value += f"{list_value},"

                    configs.append(Config(
                        type='str',
                        value=value.rstrip(','),
                        key=key,
                        service=instance
                    ))

                else:
                    configs.append(Config(
                        type=type(val).__name__ if type(val) is not None else 'str',
                        value=str(val),
                        key=key,
                        service=instance
                    ))

        Config.objects.bulk_create(configs)




from drf_auth_service.settings import settings

AUTHENTICATION_PERMISSIONS = dict(
    register=[settings.PERMISSIONS.SERVICE_TOKEN_PERMISSION],
    send_reset_password=[settings.PERMISSIONS.SERVICE_TOKEN_PERMISSION],
    reset_password_confirm=[settings.PERMISSIONS.SERVICE_TOKEN_PERMISSION],
    default=[settings.PERMISSIONS.SERVICE_TOKEN_PERMISSION]
)



from urllib.parse import urlparse

from django.utils import timezone
from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema, no_body
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.generics import GenericAPIView
from rest_framework.response import Response
from rest_framework_simplejwt.settings import api_settings
from rest_framework_simplejwt.views import TokenViewBase

from drf_auth_service.authentication.serializers import CustomTokenRefreshSerializer, \
    ReturnRegisterSerializer, ReturnAccessTokenSerializer, TokenByCookieSerializer
from drf_auth_service.common.managers import BaseManager
from drf_auth_service.common.mixins import GenericEBSViewSet
from drf_auth_service.common.register_backends import RegisterManager
from drf_auth_service.models import SSOUser
from drf_auth_service.settings import settings, User

jwt_response = openapi.Response('Respond with jwt access&refresh token', ReturnRegisterSerializer)
refresh_response = openapi.Response('Respond with jwt access token, ', ReturnAccessTokenSerializer)


class AuthenticationViewSet(GenericEBSViewSet):
    serializer_create_class = settings.SERIALIZERS.REGISTER_SERIALIZER
    serializer_class = settings.SERIALIZERS.REGISTER_RETURN_SERIALIZER
    permission_classes_by_action = settings.PERMISSIONS.AUTHENTICATION_PERMISSIONS

    @action(detail=False, methods=['POST'])
    def register(self, request, *args, **kwargs):
        api_settings.SIGNING_KEY = request.service.secret_token
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        register_manager = RegisterManager(register_type=serializer.validated_data['register_type'], request=request)
        response = register_manager.register()
        return Response(response)

    @action(detail=False, methods=['POST'], url_path='restore-request', url_name='reset-password',
            serializer_class=settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER,
            serializer_create_class=settings.SERIALIZERS.SEND_RESET_PASSWORD_SERIALIZER)
    def send_reset_password(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        manager = BaseManager.load_manager(serializer.validated_data['user'], configs=None, request=request)
        manager.send_reset_password(serializer.validated_data['user'])
        return Response(self.get_serializer(dict(message='Reset password was sent successfully')).data)

    @action(detail=False, methods=['POST'], url_path='restore-verify',
            serializer_class=settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER,
            serializer_create_class=settings.SERIALIZERS.RESET_PASSWORD_VERIFY_SERIALIZER)
    def reset_password_verify(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        message = 'Valid Token' if serializer.is_valid() else 'Invalid Token'
        return Response(self.get_serializer(
            dict(success=serializer.is_valid(), message=message)
        ).data)

    @action(detail=False, methods=['POST'], url_path='restore-confirm',
            serializer_class=settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER,
            serializer_create_class=settings.SERIALIZERS.RESET_PASSWORD_CONFIRMATION_SERIALIZER)
    def reset_password_confirm(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        password = serializer.validated_data['password']

        reset_password_token = serializer.validated_data['token']
        reset_password_token.user.set_password(password)
        reset_password_token.user.save()
        reset_password_token.delete()
        return Response(self.get_serializer(dict(message='Password was reset successfully')).data)

    @swagger_auto_schema(method='GET', request_body=no_body, responses={})
    @action(detail=False, methods=['GET'], url_name='logout')
    def logout(self, request, *args, **kwargs):
        response = Response(status=status.HTTP_200_OK)
        response.set_cookie(
            settings.COOKIE_KEY,
            "",  # Clean the token
            max_age=settings.REFRESH_TOKEN_LIFETIME.total_seconds(),
            domain=urlparse(settings.DOMAIN_ADDRESS).netloc if settings.DOMAIN_ADDRESS else None,
            httponly=True,
            secure=True
        )
        response.cookies[settings.COOKIE_KEY]['samesite'] = 'None'
        return response


class LoginViewSet(TokenViewBase):
    serializer_class = settings.SERIALIZERS.LOGIN_SERIALIZER
    permission_classes = (settings.PERMISSIONS.SERVICE_TOKEN_PERMISSION,)

    @swagger_auto_schema(request_body=settings.SERIALIZERS.LOGIN_SERIALIZER, responses={200: jwt_response})
    def post(self, request, *args, **kwargs):
        api_settings.SIGNING_KEY = request.service.secret_token
        response = super(LoginViewSet, self).post(request, *args, **kwargs)
        response.set_cookie(
            settings.COOKIE_KEY,
            response.data['refresh'],
            max_age=settings.REFRESH_TOKEN_LIFETIME.total_seconds(),
            domain=urlparse(settings.DOMAIN_ADDRESS).netloc if settings.DOMAIN_ADDRESS else None,
            httponly=True,
            secure=True
        )
        response.cookies[settings.COOKIE_KEY]['samesite'] = 'None'

        User.objects.filter(username=request.data.get('username')).update(last_login=timezone.now())

        return response


class EBSTokenRefreshView(TokenViewBase):
    serializer_class = CustomTokenRefreshSerializer
    permission_classes = (settings.PERMISSIONS.SERVICE_TOKEN_PERMISSION,)

    @swagger_auto_schema(request_body=CustomTokenRefreshSerializer, responses={200: refresh_response})
    def post(self, request, *args, **kwargs):
        api_settings.SIGNING_KEY = request.service.secret_token
        return super().post(request, *args, **kwargs)


class TokenByCookieView(GenericAPIView):
    permission_classes = ()
    serializer_class = TokenByCookieSerializer

    def get(self, request):
        refresh_token = request.COOKIES.get(settings.COOKIE_KEY)
        serializer = TokenByCookieSerializer(data={'refresh': refresh_token})
        if not serializer.is_valid():
            return Response({settings.COOKIE_KEY: ["Cookie token not valid"]}, status=status.HTTP_403_FORBIDDEN)
        return Response({'refresh': refresh_token, **serializer.validated_data}, status=status.HTTP_200_OK)




from rest_framework.permissions import IsAuthenticated

from drf_auth_service.settings import settings

USER_PERMISSIONS = dict(set_password=[settings.PERMISSIONS.SERVICE_TOKEN_PERMISSION, IsAuthenticated])



# Generated by Django 3.2.2 on 2021-05-06 03:30

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('drf_auth_service', '0003_auto_20201120_1531'),
    ]

    operations = [
        migrations.AddField(
            model_name='ssouser',
            name='created_at',
            field=models.DateTimeField(default=datetime.datetime(2021, 5, 6, 3, 30, 52, 581343)),
        ),
        migrations.AlterField(
            model_name='userlogs',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='logs', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 3.2.2 on 2021-05-06 03:40

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('drf_auth_service', '0004_auto_20210506_0330'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ssouser',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]
